#include <iostream>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <cassert>
#include <numeric>
#include <random>
using namespace std;

typedef long long ll;
typedef long double ld;

const int INF = 1e9;
const ll LINF = 1e18;
const int MOD = 1e9 + 7;

#define int long long
#define all(a) a.begin(), a.end()

struct Component {
    int l, r;
    vector <int> cmp;
    
    Component() {
        l = INF;
        r = -INF;
    }
};

struct DSU {
    vector <int> pr;
    vector <int> sz;
    
    DSU(int n) {
        pr.resize(n);
        iota(all(pr), 0);
        sz.assign(n, 1);
    }
    
    int getPr(int v) {
        if (pr[v] == v) return v;
        pr[v] = getPr(pr[v]);
        return pr[v];
    }
    
    void unionSets(int u, int v) {
        u = getPr(u);
        v = getPr(v);
        if (u == v) return;
        
        if (sz[u] < sz[v]) swap(u, v);
        sz[u] += sz[v];
        pr[v] = u;
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    
    vector <int> a(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i];
        a[i]--;
    }
    
    vector <int> l(m, INF), r(m, -1);
    for (int i = 0; i < n; i++) {
        l[a[i]] = min(l[a[i]], i);
        r[a[i]] = i;
    }
    
    DSU dsu(m);
    
    for (int i = 0; i < m; i++) {
        for (int j = i + 1; j < m; j++) {
            if (r[i] == -1) continue;
            if (r[i] < l[j] || r[j] < l[i]) continue;
            dsu.unionSets(i, j);
        }
    }
    
    vector <Component> bruh(m, Component());
    for (int i = 0; i < m; i++) {
        if (r[i] == -1) continue;
        int v = dsu.getPr(i);
        bruh[v].cmp.push_back(i);
        bruh[v].l = min(bruh[v].l, l[i]);
        bruh[v].r = max(bruh[v].r, r[i]);
    }
    
    vector <Component> comp;
    for (int i = 0; i < m; i++) {
        if (bruh[i].cmp.empty()) continue;
        comp.push_back(bruh[i]);
    }
    
    sort(all(comp), [&](Component& i, Component& j) {
        return i.l < j.l;
    });
    
    vector <vector <int>> c(m, vector <int> (m));
    for (int i = 0; i < m; i++) {
        for (int j = 0; j < m; j++) {
            cin >> c[i][j];
        }
    }
    
    int k = comp.size();
    
    vector <vector <int>> cost(k, vector <int> (m, 0));
    for (int i = 0; i < k; i++) {
        for (int v : comp[i].cmp) {
            for (int j = 0; j < m; j++) {
                cost[i][j] += c[v][j];
            }
        }
    }
    
    vector <vector <int>> dp(k, vector <int> (m));
    int mn = LINF;
    for (int i = 0; i < m; i++) {
        mn = min(mn, cost[0][i]);
        dp[0][i] = mn;
    }
    for (int i = 1; i < k; i++) {
        dp[i][0] = dp[i - 1][0] + cost[i][0];
        int mn = dp[i - 1][0] + cost[i][0];
        for (int j = 1; j < m; j++) {
            mn = min(mn, dp[i - 1][j] + cost[i][j]);
            dp[i][j] = mn;
        }
    }
    cout << dp[k - 1][m - 1] << "\n";
}

signed main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
    
    int t = 1;
    //cin >> t;
    while (t--) {
        solve();
    }
}