#include "bits/stdc++.h"
using namespace std;
using ll = long long;
const ll inf = 1e18;

random_device rd;
mt19937 rng(rd());
int rand(int l, int r) {
    return uniform_int_distribution<int>(l, r)(rng);
}


struct segtree {
    vector<ll> tree;
    int size;
    int neutral_element = 0;

    void init(int n) {
        size = 1;
        while (size < n) size *= 2;
        tree.assign(2 * size, neutral_element);
    }

    void build(const vector<int> &a) {
        size = 1;
        while (size < a.size()) size *= 2;
        tree.assign(2 * size, neutral_element);
        for (int i = size; i < size + a.size(); i++) tree[i] = a[i - size];
        for (int i = size - 1; i > 0; i--) tree[i] = merge(tree[i << 1], tree[i << 1 | 1]);
    }

    ll merge(ll a, ll b) {
        return a + b;
    }

    void upd(int pos, int val) {
        pos += size;
        tree[pos] = merge(tree[pos], val);
        for (; pos > 1; pos >>= 1) tree[pos >> 1] = merge(tree[pos], tree[pos ^ 1]);
    }

    int get(int l, int r) {
        if (l > r) return neutral_element;
        int ans = neutral_element;
        for (l += size, r += size + 1; l < r; l >>= 1, r >>= 1) {
            if (l & 1) ans = merge(ans, tree[l++]);
            if (r & 1) ans = merge(ans, tree[--r]);
        }
        return ans;
    }
};

struct fenwick {
    vector<int> tree;
    int n;

    void init(int len) {
        n = len;
        tree.assign(n + 1, 0);
    }

    void upd(int i, int v) {
        i++;
        while (i <= n) {
            tree[i] += v;
            i += i & (-i);
        }
    }

    int get(int i) {
        i++;
        int sum = 0;
        while (i > 0) {
            sum += tree[i];
            i -= i & (-i);
        }
        return sum;
    }

    int get(int l, int r) {
        return get(r) - get(l - 1);
    }
};

#define ff first
#define ss second

void solve() {
    int n;
    cin >> n;
    vector<vector<int>> g(n + 1);
    for (int i = 1; i < n; i++) {
        int u, v;
        cin >> u >> v;
        g[u].emplace_back(v);
        g[v].emplace_back(u);
    }
    int timer = 0;
    vector<int> tin(n + 1), tout(n + 1), height(n + 1, 1);
    vector<bool> vis(n + 1);
    const int lg = 19;
    vector jump(lg, vector(n + 1, 0));
    auto dfs = [&](auto &dfs, int u) -> void {
        if (u != 1) {
            for (int i = 1; i < lg; i++) {
                jump[i][u] = jump[i - 1][jump[i - 1][u]];
                if (!jump[i][u]) break;
            }
        }
        tin[u] = tout[u] = timer++;
        vis[u] = true;
        for (int v: g[u]) {
            if (vis[v]) continue;
            jump[0][v] = u;
            height[v] = height[u] + 1;
            dfs(dfs, v);
            tout[u] = tout[v];
        }
    };
    dfs(dfs, 1);
    vis = vector(n + 1, false);
    vector<ll> val(n + 1);
    fenwick cnt; cnt.init(n);
    int q, t;
    cin >> q >> t;
    ll last_ans = 0;
    // delete sum?
    while (q--) {
        int k;
        cin >> k;
        vector<int> vec(k);
        for (int &x: vec) {
            int c;
            cin >> x >> c;
            if (t == 2) x = (x + last_ans) % (n - 1) + 2;
            val[x] = c;
            vis[x] = true;
            cnt.upd(tin[x], 1);
        }
        ll ans = 0;
        while (vec.size() > 1) {
            vector<int> nw;
            for (int x: vec) {
                if (cnt.get(tin[x], tout[x]) > 1) {
                    nw.emplace_back(x);
                    continue;
                }
                ans = max(ans, height[x] * val[x]);
                int y = x;
                for (int i = lg - 1; i >= 0; i--) {
                    if (!jump[i][y]) continue;
                    if (cnt.get(tin[jump[i][y]], tout[jump[i][y]]) == 1)
                        y = jump[i][y];
                }
                y = jump[0][y];
                val[y] += val[x];
                vis[x] = val[x] = false;
                cnt.upd(tin[x], -1);
                if (!vis[y]) nw.emplace_back(y), cnt.upd(tin[y], 1);
                vis[y] = true;
            }
            if (vec.size() == nw.size()) return;
            vec = nw;
        }
        int x = vec[0];
        ans = max(ans, height[x] * val[x]);
        vis[x] = val[x] = false;
        cnt.upd(tin[x], -1);

        cout << ans << '\n';
        last_ans = ans;
    }
}

int32_t main() {
    cin.tie(0)->sync_with_stdio(false);

    int queries = 1;
#ifdef sunnatov
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
    cin >> queries;
#else
    // cin >> queries;
#endif

    for (int test_case = 1; test_case <= queries; test_case++) {
#ifdef sunnatov
        cout << "Test case: " << test_case << endl;
#endif
        solve();
        cout << '\n';
    }
}